﻿

using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static TOUT Convert<TIN, TOUT>(TIN number)
    where TOUT : struct where TIN : struct, IComparable
  {
    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 
    if (!typeof(TIN).IsPrimitive) throw new TypeAccessException(nameof(TIN));
    if (!typeof(TOUT).IsPrimitive) throw new TypeAccessException(nameof(TOUT));

    var sztsrc = SizeOfT<TIN>();
    var sztdest = SizeOfT<TOUT>();
    var minlength = Math.Min(sztsrc, sztdest);
    Span<byte> dstbytes = stackalloc byte[sztdest]; dstbytes.Clear();
    var srcbytes = MemoryMarshal.AsBytes(MemoryMarshal.CreateSpan(ref number, 1));
    for (var i = 0; i < minlength; i++) dstbytes[i] = srcbytes[i];
    if (number.CompareTo(default(TIN)) < 0 && srcbytes.Length < dstbytes.Length)
      for (var i = minlength; i < dstbytes.Length; i++) dstbytes[i] = byte.MaxValue;
    return MemoryMarshal.AsRef<TOUT>(dstbytes);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static void Convert<TIN, TOUT>(ReadOnlySpan<TIN> src, Span<TOUT> dst)
    where TIN : struct, IConvertible, IComparable
    where TOUT : struct, IConvertible
  {
    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 
    if (!typeof(TIN).IsPrimitive) throw new TypeAccessException(nameof(TIN));
    if (!typeof(TOUT).IsPrimitive) throw new TypeAccessException(nameof(TOUT));
    if (src.Length != dst.Length) throw new ArgumentOutOfRangeException(nameof(dst));
    var result = Array.ConvertAll(src.ToArray(), new Converter<TIN, TOUT>(Convert<TIN, TOUT>));
    result.CopyTo(dst);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<TOUT> Convert<TIN, TOUT>(ReadOnlySpan<TIN> src)
    where TIN : struct, IConvertible, IComparable
    where TOUT : struct, IConvertible
  {
    //New Span from Heap

    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 
    if (!typeof(TIN).IsPrimitive) throw new TypeAccessException(nameof(TIN));
    if (!typeof(TOUT).IsPrimitive) throw new TypeAccessException(nameof(TOUT));
    return Array.ConvertAll(src.ToArray(), new Converter<TIN, TOUT>(Convert<TIN, TOUT>));
  }
}

