﻿
using System.Runtime.CompilerServices;
using System.Text;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(Span<T> span, string separator = "") where T : struct
  {
    return ToString(span, 0, span.Length, separator);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(Span<T> span, int start, string separator = "") where T : struct
  {
    return ToString(span, start, span.Length - start, separator);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(Span<T> span, int start, int length, string separator = "") where T : struct
  {
    if (span.Length == 0) return string.Empty;
    if (span.Length - start - length < 0) return string.Empty;
    var result = new StringBuilder(length);
    var spanresult = span.Slice(start, length);
    for (var i = 0; i < length; i++)
      result.Append(separator + spanresult[i].ToString());
    return result.ToString();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(ReadOnlySpan<T> span, string separator = "") where T : struct
  {
    return ToString(span, 0, span.Length, separator);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(ReadOnlySpan<T> span, int start, string separator = "") where T : struct
  {
    return ToString(span, start, span.Length - start, separator);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string ToString<T>(ReadOnlySpan<T> span, int start, int length, string separator = "") where T : struct
  {
    if (span.Length == 0) return string.Empty;
    if (span.Length - start - length < 0) return string.Empty;
    var result = new StringBuilder(length);
    var spanresult = span.Slice(start, length);
    for (var i = 0; i < length; i++)
      result.Append(separator + spanresult[i].ToString());
    return result.ToString();
  }
}
