﻿

using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> AsSpan<T>(IntPtr ptr, int length)
  {
    return new Span<T>(ptr.ToPointer(), length);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> AsSpan<T>(UIntPtr uptr, int length)
  {
    return new Span<T>(uptr.ToPointer(), length);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static Span<T> ToNewSpan<T>(T value)
  {
    //with alloc from heap
    return new T[] { value };
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static ReadOnlySpan<T> ToNewReadOnlySpan<T>(T value)
  {
    //with alloc from heap
    return new T[] { value };
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static Span<T> AsSpan<T>(ref T value)
  {
    //without alloc memory
    return MemoryMarshal.CreateSpan(ref value, 1);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static ReadOnlySpan<T> AsReadOnlySpan<T>(ref T value)
  {
    //without alloc memory
    return MemoryMarshal.CreateSpan(ref value, 1);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static Span<T> AsSpan<T>(Memory<T> memory)
  {
    return memory.Span;
  }
}
