﻿using System.Runtime.CompilerServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(ReadOnlySpan<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(Span<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(ReadOnlySpan<T> span, T value, int start) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span[start..], value);
  }

  public static bool Contains<T>(Span<T> span, T value, int start) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span[start..], value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(ReadOnlySpan<T> span, T value, int start, int length) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span.Slice(start, length), value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(Span<T> span, T value, int start, int length) where T : IEquatable<T>
  {
    return MemoryExtensions.Contains(span.Slice(start, length), value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains(ReadOnlySpan<char> span, ReadOnlySpan<char> value, StringComparison comparison)
  {
    return MemoryExtensions.Contains(span, value, comparison);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(ReadOnlySpan<T> span, ReadOnlySpan<T> value) where T : IEquatable<T>
  {
    var idxs = IndexOfAll(span, value[0]);
    if (idxs.Length < 1) return false;
    if (value.Length == 1 && idxs.Length > 0) return true;

    var length = value.Length;
    for (var i = 0; i < idxs.Length; i++)
    {
      if (idxs[i] > span.Length - value.Length) return false;
      if (span.Slice(idxs[i], length).SequenceEqual(value))
        return true;
    }
    return false;
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static bool Contains<T>(Span<T> span, Span<T> value) where T : IEquatable<T>
  {
    var idxs = IndexOfAll(span, value[0]);
    if (idxs.Length < 1) return false;
    if (value.Length == 1 && idxs.Length > 0) return true;

    var length = value.Length;
    for (var i = 0; i < idxs.Length; i++)
    {
      if (idxs[i] > span.Length - value.Length) return false;
      if (span.Slice(idxs[i], length).SequenceEqual(value))
        return true;
    }
    return false;
  }
}
