﻿
using System.Runtime.CompilerServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(ReadOnlySpan<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.IndexOf(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(Span<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.IndexOf(span, value);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(Span<T> span, Span<T> value) where T : IEquatable<T>
  {
    return MemoryExtensions.IndexOf(span, value);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(ReadOnlySpan<T> span, ReadOnlySpan<T> value) where T : IEquatable<T>
  {
    return MemoryExtensions.IndexOf(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf(ReadOnlySpan<char> span, ReadOnlySpan<char> value, StringComparison comparison)
  {
    return MemoryExtensions.IndexOf(span, value, comparison);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(ReadOnlySpan<T> span, T value, int start) where T : IEquatable<T>
  {
    var result = MemoryExtensions.IndexOf(span[start..], value);
    if (result < 0) return result;
    return start + result;
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(Span<T> span, T value, int start) where T : IEquatable<T>
  {
    var result = MemoryExtensions.IndexOf(span[start..], value);
    if (result < 0) return result;
    return start + result;
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf(ReadOnlySpan<char> span, ReadOnlySpan<char> value, int start, StringComparison comparison)
  {
    var result = MemoryExtensions.IndexOf(span[start..], value, comparison);
    if (result < 0) return result;
    return start + result;
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(ReadOnlySpan<T> span, T value, int start, int length) where T : IEquatable<T>
  {
    var result = MemoryExtensions.IndexOf(span.Slice(start, length), value);
    if (result < 0) return result;
    return start + result;
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf<T>(Span<T> span, T value, int start, int length) where T : IEquatable<T>
  {
    var result = MemoryExtensions.IndexOf(span.Slice(start, length), value);
    if (result < 0) return result;
    return start + result;
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int IndexOf(ReadOnlySpan<char> span, ReadOnlySpan<char> value, int start, int length, StringComparison comparison)
  {
    var result = MemoryExtensions.IndexOf(span.Slice(start, length), value, comparison);
    if (result < 0) return result;
    return start + result;
  }

  //*********** *********** *********** *********** *********** *********** *********** *********** 
  //*********** *********** *********** *********** *********** *********** *********** *********** 


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int LastIndexOf<T>(ReadOnlySpan<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.LastIndexOf(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int LastIndexOf<T>(Span<T> span, T value) where T : IEquatable<T>
  {
    return MemoryExtensions.LastIndexOf(span, value);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int LastIndexOf(ReadOnlySpan<char> span, ReadOnlySpan<char> value, StringComparison comparison)
  {
    return MemoryExtensions.LastIndexOf(span, value, comparison);
  }

  //*********** *********** *********** *********** *********** *********** *********** *********** 
  //*********** *********** *********** *********** *********** *********** *********** *********** 

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int[] IndexOfAll<T>(ReadOnlySpan<T> span, T value) where T : IEquatable<T>
  {
    var index = -1;
    var result = new List<int>();
    while (true)
    {
      index = IndexOf(span, value, index + 1);
      if (index >= 0) result.Add(index);
      if (index < 0) break;
      if (index >= span.Length - 1) break;
    }
    return result.ToArray();
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int[] IndexOfAll<T>(Span<T> span, T value) where T : IEquatable<T>
  {
    var index = -1;
    var result = new List<int>();
    while (true)
    {
      index = IndexOf(span, value, index + 1);
      if (index >= 0) result.Add(index);
      if (index < 0) break;
      if (index >= span.Length - 1) break;
    }
    return result.ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int[] IndexOfAll<T>(Span<T> span, Span<T> value) where T : IEquatable<T>
  {
    var result = Array.Empty<int>().ToList();
    var idxs = IndexOfAll(span, value[0]);
    if (idxs.Length < 1) return Array.Empty<int>();
    if (value.Length == 1 && idxs.Length > 0) return idxs;

    var length = value.Length;
    for (var i = 0; i < idxs.Length; i++)
    {
      if (idxs[i] > span.Length - value.Length) break;
      if (span.Slice(idxs[i], length).SequenceEqual(value))
        result.Add(idxs[i]);
    }
    return result.ToArray();
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static int[] IndexOfAll(ReadOnlySpan<char> span, ReadOnlySpan<char> value, StringComparison comparison)
  {
    var index = -1;
    var result = new List<int>();
    while (true)
    {
      if (index > span.Length - value.Length) break;
      index = IndexOf(span, value, index + 1, comparison);
      if (index >= 0) result.Add(index);
      if (index < 0) break;
      if (index >= span.Length - 1) break;
    }
    return result.ToArray();
  }
}
