﻿using System.Runtime.CompilerServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Slice<T>(Span<T> span, int start)
  {
    return Slice(span, start, span.Length - start);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Slice<T>(Span<T> span, int start, int length)
  {
    if (span.Length - start - length < 0)
      throw new ArgumentOutOfRangeException(nameof(length));
    return span.Slice(start, length);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static ReadOnlySpan<T> Slice<T>(ReadOnlySpan<T> span, int start)
  {
    return Slice(span, start, span.Length - start);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static ReadOnlySpan<T> Slice<T>(ReadOnlySpan<T> span, int start, int length)
  {
    if (span.Length - start - length < 0)
      throw new ArgumentOutOfRangeException(nameof(length));
    return span.Slice(start, length);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Skip<T>(ReadOnlySpan<T> span, int start)
  {
    //Alloc Heap memory
    return span[start..].ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Take<T>(ReadOnlySpan<T> span, int length)
  {
    //Alloc Heap memory
    return span[..length].ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> SkipTake<T>(ReadOnlySpan<T> span, int start, int length)
  {
    //Alloc Heap memory
    return span.Slice(start, length).ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Skip<T>(Span<T> span, int start)
  {
    //Alloc Heap memory
    return span[start..].ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> Take<T>(Span<T> span, int length)
  {
    //Alloc Heap memory
    return span[..length].ToArray();
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<T> SkipTake<T>(Span<T> span, int start, int length)
  {
    //Alloc Heap memory
    return span.Slice(start, length).ToArray();
  }

}
