﻿using System.Runtime.CompilerServices;
using System.Text;

namespace exc.jdbi.Spans;


partial class SpanTool
{

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static Span<char> SubString(Span<char> str, int start)
  {
    return str[start..];
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static Span<char> SubString(Span<char> str, int start, int length)
  {
    return str.Slice(start, length);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static void Insert(Span<char> str, char ch, int index)
  {
    str[index] = ch;
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static void Insert(Span<char> str, string values, int start)
  {
    if (str.Length - start - values.Length < 0)
      throw new ArgumentOutOfRangeException(nameof(values));
    values.CopyTo(str.Slice(start, values.Length));
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Insert(ReadOnlySpan<char> str, char ch, int index)
  {
    var result = str.ToArray();
    Insert(result, ch, index);
    return ToString<char>(result);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Insert(ReadOnlySpan<char> str, string values, int start)
  {
    if (str.Length - start - values.Length < 0)
      throw new ArgumentOutOfRangeException(nameof(values));
    var result = str.ToArray();
    Insert(result, values, start);
    return ToString<char>(result);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, char oldch, char newch)
  {
    return Replace(str, oldch, newch, 0, str.Length);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, char oldch, string newvalues)
  {
    return Replace(str, oldch, newvalues, 0, str.Length);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, string oldvalues, string newvalues)
  {
    return Replace(str, oldvalues, newvalues, 0, str.Length);
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, char oldch, char newch, int start, int length)
  {
    if (oldch == newch) return ToString(str);
    if (str.Length - start - length < 0)
      throw new ArgumentOutOfRangeException(nameof(str));
    var result = str.ToArray().AsSpan();
    var tmp = result.Slice(start, length);
    for (var i = 0; i < tmp.Length; i++)
      if (tmp[i] == oldch) tmp[i] = newch;
    return ToString(result);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, char oldvalue, string newvalues, int start, int length)
  {
    if (oldvalue.ToString() == newvalues) return ToString(str);
    if (str.Length - start - length < 0)
      throw new ArgumentOutOfRangeException(nameof(str));
    var result = new StringBuilder(str.Length);
    result.Append(str[..start]);

    for (var i = start; i <= start + length - 1; i++)
      if (str[i] == oldvalue) result.Append(newvalues);
      else result.Append(str[i]);
    result.Append(str.Slice(start + length, str.Length - start - length));
    return ToString<char>(result.ToString());
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static string Replace(ReadOnlySpan<char> str, string oldvalues, string newvalues, int start, int length)
  {
    if (oldvalues.SequenceEqual(newvalues)) return ToString(str);
    if (oldvalues.Length == 1) return Replace(str, oldvalues[0], newvalues);

    if (str.Length - start - length < 0)
      throw new ArgumentOutOfRangeException(nameof(str));
    var result = new StringBuilder(str.Length);
    result.Append(str[..start]);
    var tmp = str.Slice(start, length);
    var indexes = IndexOfAll(tmp, oldvalues, StringComparison.InvariantCulture);

    var idx = 0;
    for (var i = start; i <= start + length - oldvalues.Length + 1; i++)
    {
      if (indexes[idx] != i) result.Append(str[i]);
      else
      {
        result.Append(newvalues);
        i += oldvalues.Length - 1;
        idx++;
        if (idx >= indexes.Length) idx = 0;
      }
    }
    result.Append(str.Slice(start + length, str.Length - start - length));
    return ToString<char>(result.ToString());

  }
}
