﻿


using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

namespace exc.jdbi.Spans;


partial class SpanTool
{
  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public static TOUT ConvertToUnmanaged<TIN, TOUT>(in TIN number)
    where TOUT : unmanaged where TIN : unmanaged, IComparable
  {
    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 

    //if (!typeof(TIN).IsPrimitive) throw new TypeAccessException(nameof(TIN));
    //if (!typeof(TOUT).IsPrimitive) throw new TypeAccessException(nameof(TOUT));
    var sztsrc = SizeOfTUnmanaged<TIN>();
    var sztdest = SizeOfTUnmanaged<TOUT>();
    var minlength = Math.Min(sztsrc, sztdest);
    Span<byte> dstbytes = stackalloc byte[sztdest]; dstbytes.Clear();

    var _number = number;
    var srcbytes = MemoryMarshal.AsBytes(MemoryMarshal.CreateSpan(ref _number, 1));
    for (var i = 0; i < minlength; i++) dstbytes[i] = srcbytes[i];
    if (number.CompareTo(default(TIN)) < 0 && srcbytes.Length < dstbytes.Length)
      for (var i = minlength; i < dstbytes.Length; i++) dstbytes[i] = byte.MaxValue;
    return MemoryMarshal.AsRef<TOUT>(dstbytes);
  }

  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static void ConvertUnmanaged<TIN, TOUT>(in ReadOnlySpan<TIN> src, in Span<TOUT> dst)
    where TIN : unmanaged where TOUT : unmanaged
  {
    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 
    if (src.Length != dst.Length) throw new ArgumentOutOfRangeException(nameof(dst));

    var sztsrc = sizeof(TIN);
    var sztdest = sizeof(TOUT);

    fixed (TIN* ptrsrc = &src[0])
    fixed (TOUT* ptrdst = &dst[0])
    {
      var length = src.Length;
      var ptrbytessrc = (byte*)ptrsrc;
      var ptrbytesdst = (byte*)ptrdst;
      var szsrcbytes = sztsrc * src.Length;
      var szdstbytes = sztdest * dst.Length;
      var minlength = Math.Min(sztsrc, sztdest);
      for (int i = 0, j = 0, k = 0; i < length; i++, j += sztsrc, k += sztdest)
      {
        for (var l = 0; l < minlength; l++)
          *(ptrbytesdst + l + k) = *(ptrbytessrc + l + j);
      }
    }
  }


  [SkipLocalsInit]
  [MethodImpl(MethodImplOptions.AggressiveInlining)]
  public unsafe static Span<TOUT> ConvertUnmanaged<TIN, TOUT>(in ReadOnlySpan<TIN> src)
      where TIN : unmanaged where TOUT : unmanaged
  {
    //New Span alloc from Heap

    //Die Zahlen einer Menge (Array) werden einzeln betrachtet. 
    //The numbers of a set (Array) are considered individually. 
    var result = new TOUT[src.Length].AsSpan();
    ConvertUnmanaged(src, result);
    return result;
  }
}
