

Imports System.Text
Imports exc.jdbi.Spans
Imports exc.jdbi.Pointers
Imports System.Runtime.CompilerServices

Namespace SpanToolTest

  Friend Module TestSpanTool

    Public Sub Start()
      TestSizeOfT()
      TestAsArray()
      TestAsRef()
      TestCast()
      TestConvert()
      TestCopy()
      TestToString()
      TestToAsSpan()
      TestBinarySearch()
      TestClear()
      TestConcat()
      TestContains()
      TestCreateInstance()
      TestEmpty()
      TestFill()
      TestForEach()
      TestIndexOf()
      TestReverse()
      TestSlice()
      TestStrings()
      TestSwap()
      TestToUpperLower()
      TestPointers()
    End Sub

    Private Sub TestSizeOfT()
      Dim size = SpanTool.SizeOfT(Of UInt32)()
    End Sub


    Private Sub TestAsArray()
      ' Wichtig: Der gleiche Speicherbereich wird zurckgegeben
      ' Important: The same memory area Is returned.

      ' Wichtig: Wird die Source verndert, ndert sich 
      ' auch die Byte-Destination !

      ' Es wird kein neuer Speicher alloziert.

      Dim bytes = RngBytes(Rand.Next(20, 100))
      Dim span1 = SpanTool.AsBytes(Of Byte)(bytes)
      span1(0) = 1 ' In bytes ndert es sich auch
      bytes(0) = 10 ' genauso auch jetzt in span1

      Dim uints = RngInts(Of UInt32)(Rand.Next(20, 100))
      Dim span2 = SpanTool.AsBytes(Of UInt32)(uints)
      uints(0) = 1 'siehe in span2

    End Sub

    Private Sub TestAsRef()
      ' Zurzeit ist in Vb.Net der Rckgabewert einer Funktion
      ' als 'ByRef' nicht mglich. (ber Parameterliste schon)

      'In C# hingegen kann mit 'ref' eine referenzierte
      'Deklaration gemacht werden, die im Ursprung auch ndert.

      ' Der zurckgegebene Wert aus 'AsRef' wird in eine
      ' neue Variable auf dem Stack geschrieben.

      Dim bytes = RngBytes(Rand.[Next](5, 10)).AsSpan()
      Dim spanbytes = SpanTool.AsRef(Of Byte)(bytes)
      spanbytes = 2 ' In bytes ndert sich nichts. 

      Dim spani16 = SpanTool.AsRef(Of Int16)(bytes)
      spani16 = 2 ' In bytes ndert sich nichts.

    End Sub

    Private Sub TestCast()
      ' Wichtig: Der Cast von einem Typ in den anderen, kann unter Umstnden 
      ' problematisch sein, da der gleiche Speicherbereich genutzt 
      ' wird, und somit die Byte-Zahl nicht bereinstimmt muss. 
      ' Beispiel: Int16 zu Int32, wenn Int16 size = 51 ist.
      ' Da 51 * 2 mod 4 != 0 ist

      ' SpanTool besitzt bei Cast die option 'cut_admit'
      ' Der Basiswert ist false, und gibt im oberen Beispiel ein Exception aus.
      ' Sofern true wird, einfach abgeschnitten! >> 51 * 2 div 4 = 25,
      ' da nur der jeweilige Speicherbereich genutzt werden kann.

      ' ******** ******** ******** ******** ******** ******** ******** 
      ' Sofern das nicht erwnscht ist die Copy-Methoden nutzen.
      ' ******** ******** ******** ******** ******** ******** ******** 


      Dim uints = RngInts(Of UInt32)(Rand.Next(5, 10))
      Dim spanbytes = SpanTool.Cast(Of UInt32, Byte)(uints)
      uints(0) = 1 ' spanbytes ndert sich, da der gleiche Speicherbereich


      Dim i16s = RngInts(Of Int16)(51)
      ' Exception wenn size = 51 und cut_admit = false
      Dim spanints = SpanTool.Cast(Of Int16, Int32)(i16s, True) ' Wichtig: size = 25
      'spanints[0] = 1; // i16s wrde sich ndert


      ' Die Copy-Methode lst das Problem, wobei hier zwei verschiedene 
      ' Speicherbereiche (hier beide im Heap) genutzt werden.
      Dim ints = New Int32(25) {} ' 51 * 2 / 4 --> 26
      SpanTool.Copy(Of Int16, Int32)(i16s, 0, ints, 0, i16s.Length)


      'spanints != ints

    End Sub

    Private Sub TestConvert()

      ' Bei einer Konvertierung werden in einer Array mit
      ' primitiven Datentypen, die einzelnen Werte nicht als
      ' Gesamtwert (zusammenhngend) betrachtet, sondern
      ' jedes einzelne Item (Zahl) wird in ein bestimmter
      ' Datentyp konvertiert. 
      ' >> Anders als bei Copy(..)


      Dim uints1 = RngInts(Of UInt32)(Rand.[Next](5, 10))
      Dim spani16s = SpanTool.Convert(Of UInt32, Int16)(uints1)

      'Mit 'Array.ConvertAll' kann es so gemacht werden
      Dim converter = New Converter(Of UInt32, Int16)(AddressOf ToT)
      Dim i16s = Array.ConvertAll(uints1, converter)
      Dim uints2 = Array.ConvertAll(i16s, New Converter(Of Int16, UInt32)(AddressOf ToT))

      'uints1 == uints2 genauso auch spani16s == i16s


      'Hier noch eine Char-Variante
      Dim chars = RngInts(Of Char)(Rand.[Next](10, 20))
      Dim spanuints = SpanTool.Convert(Of Char, UInt32)(chars)
    End Sub


    Private Sub TestCopy()
      'Copy eignet sich hervorragend, wenn verschiedene 
      'Speicherbereiche vorhanden sind (auch in der Lnge). 
      'Es knnen Heap, Stack, Unmanaged Speicherbereiche verwendet werden.

      Dim i16s = RngInts(Of Int16)(Rand.[Next](5, 10))
      Dim size = If(i16s.Length * SpanTool.SizeOfT(Of Int16)() Mod SpanTool.SizeOfT(Of Int32)() = 0, i16s.Length * SpanTool.SizeOfT(Of Int16)() / SpanTool.SizeOfT(Of Int32)(), (i16s.Length * SpanTool.SizeOfT(Of Int16)() / SpanTool.SizeOfT(Of Int32)()) + 1)
      Dim ints = New Int32(size - 1) {}
      SpanTool.Copy(Of Int16, Int32)(i16s, 0, ints, 0, i16s.Length)


      Dim uints1 = RngInts(Of UInt32)(Rand.[Next](5, 10))
      Dim uints2 = New UInt32(uints1.Length - 1) {}
      SpanTool.Copy(Of UInt32)(uints1, 0, uints2, 0, uints1.Length)

    End Sub

    Private Sub TestToString()
      'Macht, was gemacht werden muss

      Dim bytes = RngBytes(Rand.Next(5, 20))
      Dim str = SpanTool.ToString(Of Byte)(bytes, " ")

      Dim uints = RngInts(Of UInt32)(Rand.Next(5, 20))
      str = SpanTool.ToString(Of UInt32)(uints, " ")

      Dim i16s = RngInts(Of Int16)(Rand.Next(5, 20))
      str = SpanTool.ToString(Of Int16)(i16s, " ")

    End Sub

    Private Sub TestToAsSpan()
      Dim number = RngInt(Of Int32)()

      'Der gleiche Speicherbereich wird zurckgegeben.
      Dim spanref = SpanTool.AsSpan(number)
      spanref(0) = 123456 'number ndert sich


      Dim span = SpanTool.ToNewSpan(number)
      span(0) = 654321 'number ndert sich

    End Sub

    Private Sub TestBinarySearch()
      'https://github.com/dotnet/runtime/issues/20107
      'https://www.geeksforgeeks.org/how-to-use-array-binarysearch-method-in-c-sharp-set-1/


      ' Sucht den gewnschten Value in einer >sortierten< Span
      ' >> hnlich wie Contains

      Dim i32value = RngInt(Of Int32)()
      Dim i32s = RngInts(Of Int32)(10)
      Dim i32ssort = i32s.ToArray()

      ' Array muss sortiert sein
      Array.Sort(i32ssort)
      Dim index = i32ssort.AsSpan().BinarySearch(i32value) 'Nicht gefunden

      Dim comparer = New InlineableComparable()
      Dim cntval = i32s.Count(Function(x) x = i32value)
      index = SpanTool.BinarySearch(Of Int32)(i32ssort, comparer) 'Nicht gefunden


      i32value = i32s(Rand.[Next](i32s.Length))
      comparer = New InlineableComparable(i32value)
      cntval = i32s.Count(Function(x) x = i32value)
      index = SpanTool.BinarySearch(Of Int32)(i32ssort, comparer)  ' gefunden
    End Sub


    Private Sub TestClear()
      Dim uints = RngInts(Of UInt32)(Rand.Next(5, 20))
      SpanTool.Clear(Of UInt32)(uints)
    End Sub

    Private Sub TestConcat()
      'Der neue Speicher fr die Zusammensetzung
      'kommt immer aus dem Heap


      Dim bytes1 = RngBytes(Rand.Next(10, 30))
      Dim bytes2 = RngBytes(Rand.Next(10, 30))
      Dim concat = SpanTool.Concat(Of Byte)(bytes1, bytes2)
    End Sub


    Private Sub TestContains()
      Dim i16value = RngInt(Of Int16)()
      Dim i16s = RngInts(Of Int16)(Rand.[Next](10, 30))
      i16s(Rand.[Next](i16s.Length)) = i16value
      Dim result = SpanTool.Contains(Of Int16)(i16s, i16value)
      'var index = SpanTool.IndexOf(i16s, i16value);

      Dim chs = RngAlphaString(Rand.[Next](10, 30))
      Dim chvalue = chs(Rand.[Next](chs.Length))
      result = SpanTool.Contains(Of Char)(chs, chvalue)
      'index = SpanTool.IndexOf(chs, chvalue);

      result = SpanTool.Contains(chs, {chvalue}, StringComparison.CurrentCulture)
      'index = SpanTool.IndexOf(chs, new[] { chvalue }, StringComparison.CurrentCulture);

      chs = RngAlphaString(Rand.[Next](10, 30))
      Dim start = Rand.[Next](chs.Length / 3)
      Dim length = Rand.[Next](3, 10)
      Dim chvalues = chs.AsSpan().Slice(start, length).ToArray()
      result = SpanTool.Contains(chs, chvalues, StringComparison.CurrentCulture)
      'index = SpanTool.IndexOf(chs, chvalues, StringComparison.CurrentCulture);
    End Sub

    Private Sub TestCreateInstance()
      Dim spanuints = SpanTool.CreateInstance(Of UInt32)(32)
      spanuints = CType(SpanTool.CreateInstance(GetType(UInt32), 32), UInt32())

      Dim spanuintsjagged = SpanTool.CreateInstance(Of UInt32)(8, 16, 32, 64)

      'false for jagged
      Dim tmp = CType(SpanTool.CreateInstance(GetType(UInt32), {8, 16, 32, 64}, False), Array())
      spanuintsjagged = tmp.[Select](Function(x) CType(x, UInt32())).ToArray()

    End Sub

    Private Sub TestEmpty()
      Dim span = SpanTool.Empty(Of UInt32)()

    End Sub

    Private Sub TestFill()
      Dim uints = New UInt32(9) {}
      SpanTool.Fill(Of UInt32)(uints, UInt32.MaxValue)
    End Sub

    Private Sub TestForEach()
      Dim i16s = RngInts(Of Int16)(Rand.[Next](10, 30))
      SpanTool.ForEach(Of Int16)(i16s, Sub(x) Console.Write($"{x} "))
      Console.WriteLine()

      SpanTool.ForEach(Of Int16)(i16s, Sub(x, idx) Console.Write(If(idx Mod 2 = 0, $"{x} ", String.Empty)))
      Console.WriteLine()
    End Sub

    Private Sub TestIndexOf()
      Dim bvalue = RngInt(Of Byte)()
      Dim bytes = RngBytes(Rand.[Next](20, 50))
      Dim result = SpanTool.IndexOf(Of Byte)(bytes, bvalue)


      Dim i16value = RngInt(Of Int16)()
      Dim i16s = RngInts(Of Int16)(Rand.[Next](20, 50))
      i16s(Rand.[Next](i16s.Length)) = i16value
      result = SpanTool.IndexOf(Of Int16)(i16s, i16value)


      Dim i32value = RngInts(Of Int32)(3)
      Dim i32s = Enumerable.Repeat(1, 100).ToArray()
      result = SpanTool.IndexOf(Of Int32)(i32s, i32value)
      i32value.AsSpan().CopyTo(i32s.AsSpan().Slice(Rand.[Next](i32s.Length / 3), i32value.Length))
      result = SpanTool.IndexOf(Of Int32)(i32s, i32value)


      bvalue = RngInt(Of Byte)()
      bytes = RngBytes(Rand.[Next](128, 1000))
      Dim resultall = SpanTool.IndexOfAll(Of Byte)(bytes, bvalue)
      'var rcntall = bytes.Count(x => x == bvalue);


      'und noch einige andere Mglichkeiten...

    End Sub


    Private Sub TestReverse()

      Dim bytes1 = RngBytes(Rand.Next(10, 20))
      SpanTool.Reverse(Of Byte)(bytes1)

      bytes1 = RngBytes(Rand.Next(10, 20))
      Dim start = Rand.Next(2, 5)
      SpanTool.Reverse(Of Byte)(bytes1, start)

      bytes1 = RngBytes(Rand.Next(10, 20))
      Dim length = Rand.Next(3, 6)
      SpanTool.Reverse(Of Byte)(bytes1, start, length)

    End Sub


    Private Sub TestSlice()
      Dim bytes = RngBytes(Rand.[Next](10, 20))
      Dim slice1 = bytes.AsSpan().Slice(2, 5)
      Dim slice2 = SpanTool.Slice(Of Byte)(bytes, 2, 5)

      'Alloc memory from heap
      Dim skip = SpanTool.Skip(Of Byte)(bytes, 2)
      Dim take = SpanTool.Take(Of Byte)(bytes, 5)
      Dim skiptake = SpanTool.SkipTake(Of Byte)(bytes, 2, 5)
    End Sub


    Private Sub TestStrings()
      Dim s = "exc-jdbi"
      Dim str = RngAlphaString(Rand.[Next](128, 513))
      Dim strspan = str.ToCharArray().AsSpan()
      SpanTool.Insert(strspan, " "c, 0)
      SpanTool.Insert(strspan, s, 0)

      Dim newstr = SpanTool.Insert(str, " "c, 0)
      newstr = SpanTool.Insert(str, s, 0)

      str = New String("a"c, 10)
      newstr = SpanTool.Replace(str, "b"c, "a"c)
      newstr = SpanTool.Replace(str, "a"c, "b"c)

      str = "acacacacacacacacacac"
      newstr = SpanTool.Replace(str, "b"c, "a"c)
      newstr = SpanTool.Replace(str, "a"c, "b"c)
      newstr = SpanTool.Replace(str, "a"c, s)
      newstr = SpanTool.Replace(str, "a", s)

      str = "akackackackackackackackackaaca"
      newstr = SpanTool.Replace(str, "ac", s, 0, 29)
    End Sub

    Private Sub TestSwap()
      Dim bytes = RngBytes(Rand.Next(10, 20))
      SpanTool.Swap(Of Byte)(bytes, 1, 3)
    End Sub


    Private Sub TestToUpperLower()
      'new alloc memory from heap
      Dim chs = RngAlphaString(Rand.Next(5, 10))
      Dim lower = SpanTool.ToLower(chs)
      Dim upper = SpanTool.ToUpper(chs)
    End Sub


    Private Function ToT(Of TIN As {Structure, IConvertible, IComparable}, TOUT As {Structure, IConvertible})(number As TIN) As TOUT
      Return SpanTool.Convert(Of TIN, TOUT)(number)
    End Function

    Private Function RngAlphaString(size As Int32) As String
      Dim alpha = AlphaString().OrderBy(Function(c) Rand.[Next]()).ToArray()
      Dim result = New StringBuilder(size)
      Dim i = 0, loopTo = size - 1

      While i <= loopTo
        result.Append(alpha(Rand.[Next](alpha.Length)))
        i += 1
      End While
      Return result.ToString()
    End Function

    Private Function AlphaString() As String
      Dim result = New StringBuilder(62)
      Dim z = New Byte() {48, 57, 65, 90, 97, 122}
      Dim i = 0, loopTo = z.Length - 1

      While i <= loopTo
        Dim j = z(i), loopTo1 = z(i + 1)

        While j <= loopTo1
          result.Append(Convert.ToChar(j))
          j += 1
        End While

        i += 2
      End While
      Return result.ToString()
    End Function

    Private Sub TestPointers()

      'Die Zahlen in dieser Array werden verndert,
      'gleichzeitig ist die Array wie "fixiert"
      Dim uints = RngInts(Of UInt32)(4)

      Using gcptr = New GcIPtr(Of UInt32)(uints)
        Dim target = gcptr.Target

        Dim copybytes = gcptr.ToArray
        copybytes(1) = 3

        Dim copyvalues = gcptr.ToValues
        copyvalues(1) = 3

        target(0) = 5
        target(2) = 5

        Dim ptr = gcptr.Ptr

        Dim value = gcptr.Target(2)
        value = gcptr.Target(2)
      End Using

      'Die Array wird in den unmanaged Bereich
      'kopiert. Im Unmanaged ist die Array wie "fixiert".
      uints = RngInts(Of UInt32)(4)

      'UsIPtr bietet minimale Funktionen,
      'die sonst in der Praxis gebraucht werden.
      Using usptr = New UsIPtr(Of UInt32)(uints)
        Dim value = usptr.Target(2)

        Dim target = usptr.Target
        target(2) = 5
        usptr(0) = 5

        Dim ptr = usptr.Ptr

        Dim copybytes = usptr.ToBytes
        copybytes(1) = 2

        Dim copyvalues = usptr.ToValues
        copyvalues(1) = 1
      End Using


      'Und nochmals GcIPtr
      uints = RngInts(Of UInt32)(4)
      Dim gcptr2 = New GcIPtr(Of UInt32)(uints)
      Dim target2 = gcptr2.Target
      Dim copybytes2 = gcptr2.ToArray

      target2(0) = 5
      target2(2) = 5

      Dim ptr2 = gcptr2.Ptr

      Dim value2 = gcptr2.Target(2)
      gcptr2.Dispose()

      'Und nochmals UsIPtr
      uints = RngInts(Of UInt32)(4)
      Dim usptr2 = New UsIPtr(Of UInt32)(uints)
      value2 = usptr2.Target(2)

      Dim target3 = usptr2.Target
      target3(2) = 5
      usptr2(0) = 5

      ptr2 = usptr2.Ptr

      copybytes2 = usptr2.ToBytes
      Dim copyvalues2 = usptr2.ToValues
      usptr2.Dispose()

    End Sub

  End Module



  Partial Friend Structure InlineableComparable
    Implements IComparable(Of Int32)

    Private ReadOnly m_value As Int32

    Public Sub New(value As Int32)
      Me.m_value = value
    End Sub

    <MethodImpl(MethodImplOptions.AggressiveInlining)>
    Public Function CompareTo(other As Int32) As Int32 Implements IComparable(Of Int32).CompareTo
      If Me.m_value = other Then Return 0
      If Me.m_value < other Then Return -1
      Return 1
    End Function
  End Structure
End Namespace
